"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDefaultModel = createDefaultModel;
exports.model = model;
exports.validate = exports.normalize = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _isEmpty = _interopRequireDefault(require("lodash/isEmpty"));

var _defaults = _interopRequireDefault(require("./defaults"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

var getContent = function getContent(html) {
  return (html || '').replace(/(<(?!img|iframe|source)([^>]+)>)/gi, '');
};

var checkNullish = function checkNullish(value) {
  return value !== null && value !== undefined;
};

var isEnabled = function isEnabled(value, defaultValue) {
  return checkNullish(value) ? value : defaultValue;
};

function createDefaultModel() {
  var model = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return new Promise(function (resolve) {
    resolve(_objectSpread(_objectSpread({}, _defaults["default"]), model));
  });
}

var normalize = function normalize(question) {
  return _objectSpread(_objectSpread({}, _defaults["default"]), question);
};

exports.normalize = normalize;

var normalizeTeacherInstructions = function normalizeTeacherInstructions(instructions, env) {
  var role = env.role,
      mode = env.mode;

  if (role === 'instructor' && (mode === 'view' || mode === 'evaluate')) {
    return instructions || '';
  }

  return '';
};
/**
 *
 * @param {*} question
 * @param {*} session
 * @param {*} env
 */


function model(_x, _x2, _x3) {
  return _model.apply(this, arguments);
}

function _model() {
  _model = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(question, session, env) {
    var normalizedQuestion, normalizedPassages;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            normalizedQuestion = normalize(question);
            normalizedPassages = normalizedQuestion.passages.map(function (passage, index) {
              return {
                teacherInstructions: isEnabled(passage.teacherInstructionsEnabled, normalizedQuestion.teacherInstructionsEnabled) ? normalizeTeacherInstructions(passage.teacherInstructions, env) : '',
                label: getContent(passage.title) ? passage.title : "Passage ".concat(index + 1),
                title: isEnabled(passage.titleEnabled, normalizedQuestion.titleEnabled) ? passage.title || '' : '',
                subtitle: isEnabled(passage.subtitleEnabled, normalizedQuestion.subtitleEnabled) ? passage.subtitle || '' : '',
                author: isEnabled(passage.authorEnabled, normalizedQuestion.authorEnabled) ? passage.author || '' : '',
                text: isEnabled(passage.textEnabled, normalizedQuestion.textEnabled) ? passage.text || '' : ''
              };
            });
            return _context.abrupt("return", _objectSpread(_objectSpread({}, normalizedQuestion), {}, {
              passages: normalizedPassages
            }));

          case 3:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _model.apply(this, arguments);
}

var validate = function validate() {
  var model = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var errors = {};
  var passagesErrors = {};
  (model.passages || []).forEach(function (passage, index) {
    var err = {};
    ['teacherInstructions', 'title', 'subtitle', 'author', 'text'].forEach(function (field) {
      var _config$field;

      if ((_config$field = config[field]) !== null && _config$field !== void 0 && _config$field.required && !getContent(passage[field])) {
        err[field] = 'This field is required.';
      }
    });

    if (!(0, _isEmpty["default"])(err)) {
      passagesErrors[index] = err;
    }
  });

  if (!(0, _isEmpty["default"])(passagesErrors)) {
    errors.passages = passagesErrors;
  }

  return errors;
};

exports.validate = validate;
//# sourceMappingURL=index.js.map